/*global window, rJS, jIO, FormData */
/*jslint indent: 2, maxerr: 3 */


(function (window, rJS) {
  "use strict";

  /*
    Next two functions deal with a weird issue of unlin-ed strings, ie. list
    of strings instead a single joined string. See 
    https://github.com/jupyter/jupyter-drive/blob/master/jupyterdrive/gdrive/notebook_model.js#L45
  */
  function unsplit_lines(multiline_string) {
    if (Array.isArray(multiline_string)) {
      return multiline_string.join('');
    } else {
      return multiline_string;
    }
  }
  
  function transform_notebook(notebook, transform_fn) {
    if (!notebook['cells']) {
      return null;
    }
    notebook['cells'].forEach(function (cell) {
      if (cell['source']) {
        cell['source'] = transform_fn(cell['source']);
      }
      if (cell['outputs']) {
        cell['outputs'].forEach(function (output) {
          if (output['data']) {
            output['data'] = transform_fn(output['data']);
          }
        });
      }
    });
  }

  function toNotebookModel(id, obj, keepContent) {
    //console.log(obj.value.text_content);
    var cont = null;
    if(keepContent) {
      cont = JSON.parse(obj.text_content);
      transform_notebook(cont, unsplit_lines);
      cont.metadata = cont.metadata || {};
    }
    var nbobj = { "name": obj.title,
                  "path": id,
                  "type": "notebook",
                  "format": "json",
                  "writable": true,
                  "mimetype": "application/ipynb",
                  "content": cont,
                  //"content": keepContent ? obj.text_content : null
                };
    return nbobj;
  }

  function handleEvent(e) {
    if(e.detail.path === "") {
      this.jio_allDocs({
          query: 'portal_type: "Web JSON"',
          select_list : ['text_content', 'title', 'reference'], //, 'modification_date'], NOT WORKING
          // sort_on: [["modification_date", "descending"]], // NOT WORKING
          limit: [0, 5]
        })
        .push(function(result) {
          var nbs = [];
          for(var i = 0; i < result.data.rows.length; i++) {
            nbs.push(toNotebookModel(result.data.rows[i].id, result.data.rows[i].value, false));
          }
          var root_dir = {"name": "",
                          "path": "",
                          "last_modified": "2017-09-06T03:33:29.781159Z",
                          "created": "2017-09-06T03:33:29.781159Z",
                          "content": nbs,
                         };
          e.detail.resolve(root_dir);
        }, function(err) {
          e.detail.reject();
          console.log(err);
        });
    } else {
      this.jio_get(e.detail.path)
      .push(function(result) {
        e.detail.resolve(toNotebookModel(e.detail.path, result, true));
      }, function(err) {
        console.log(err);
      });
    }
  }

  rJS(window)
    .ready(function (gadget) {
      console.log("Jio Contents Gadget yes yes.");
      return gadget;
    })
    .declareAcquiredMethod("jio_allDocs", "jio_allDocs")
    .declareAcquiredMethod("jio_get", "jio_get")
    .onEvent("custom_event", handleEvent, false, true)
    .declareMethod('render', function () {
      console.log("Rendering!");
      document.jiocontentsReady = true;
      return this;
    });

}(window, rJS));

